@@ -8,7 +8,7 @@
 
 #include "globals.h"
 #include <string.h>
-#include <curses.h>
+#include <ncursesw/curses.h>
 #include <wctype.h>
 #include <sys/time.h>
 #include <time.h>
