@@ -1,9 +1,9 @@
-/* © 2007 David Given.
+/* © 2008 David Given.
  * WordGrinder is licensed under the BSD open source license. See the COPYING
  * file in this distribution for the full text.
  *
- * $Id: main.c 33 2007-10-23 22:35:46Z dtrg $
- * $URL: $
+ * $Id: main.c 121 2008-09-07 15:35:27Z dtrg $
+ * $URL: https://wordgrinder.svn.sf.net/svnroot/wordgrinder/wordgrinder/src/c/main.c $
  */
 
 #include <stdlib.h>
@@ -16,12 +16,13 @@ int main(int argc, const char* argv[])
 	setlocale(LC_ALL, "");
 	setlocale(LC_COLLATE, "C");
 	script_init();
-	screen_init();
+	screen_init(argv);
 	word_init();
 	bit_init();
 	utils_init();
-	
-	script_load(LUA_SRC_DIR "main.lua", argv);
-	
+
+	script_load_from_table(script_table);
+	script_run(argv);
+
 	return 0;
 }
