@@ -1,9 +1,9 @@
--- © 2007 David Given.
+-- © 2008 David Given.
 -- WordGrinder is licensed under the BSD open source license. See the COPYING
 -- file in this distribution for the full text.
 --
--- $Id: fileio.lua 46 2008-01-13 06:39:30Z dtrg $
--- $URL: https://wordgrinder.svn.sourceforge.net/svnroot/wordgrinder/wordgrinder/src/lua/fileio.lua $
+-- $Id: fileio.lua 121 2008-09-07 15:35:27Z dtrg $
+-- $URL: https://wordgrinder.svn.sf.net/svnroot/wordgrinder/wordgrinder/src/lua/fileio.lua $
 
 local ITALIC = wg.ITALIC
 local UNDERLINE = wg.UNDERLINE
@@ -73,12 +73,10 @@ local function writetostream(object, fp)
 	return true
 end
 
-local function savedocument(filename)
-	ImmediateMessage("Saving...")
-	
-	local fp = io.open(filename, "w")
+function SaveDocumentSetRaw(filename)
+	local fp, e = io.open(filename, "wb")
 	if not fp then
-		return false
+		return false, e
 	end
 	
 	DocumentSet:purge()
@@ -98,9 +96,11 @@ function Cmd.SaveCurrentDocumentAs(filename)
 		DocumentSet.name = filename
 	end
 
-	local r = savedocument(DocumentSet.name)
+	ImmediateMessage("Saving...")
+	DocumentSet:clean()	
+	local r, e = SaveDocumentSetRaw(DocumentSet.name)
 	if not r then
-		ModalMessage("Save failed", "The document could not be saved for some reason.")
+		ModalMessage("Save failed", "The document could not be saved: "..e)
 	else
 		NonmodalMessage("Save succeeded.")
 	end
@@ -230,7 +230,7 @@ local function loadfromstream(fp)
 end
 
 local function loaddocument(filename)
-	local fp, e = io.open(filename)
+	local fp, e = io.open(filename, "rb")
 	if not fp then
 		return nil, ("'"..filename.."' could not be opened: "..e)
 	end
@@ -250,6 +250,10 @@ local function loaddocument(filename)
 	
 	d:purge()
 	
+	-- Even if the changed flag was set in the document on disk, remove it.
+	
+	d:clean()
+	
 	d.name = filename
 	return d
 end
@@ -296,21 +300,27 @@ do
 	local function cb(event, token)	
 		local fileformat = DocumentSet.fileformat or 1
 		
-		if (fileformat == FILEFORMAT) then
-			return
+		if (fileformat ~= FILEFORMAT) then
+			ModalMessage("Upgrading document",
+				"You are trying to open a file belonging to an earlier "..
+				"version of WordGrinder. That's not a problem, but if you "..
+				"save the file again it may not work on the old version. "..
+				"Also, all keybindings defined in this file will get reset "..
+				"to their default values.")
+			
+			ImmediateMessage("Upgrading...")
+			FireEvent(Event.DocumentUpgrade, fileformat, FILEFORMAT)
+				
+			DocumentSet.fileformat = FILEFORMAT
+			DocumentSet.menu = CreateMenu()
+			DocumentSet:touch()
 		end
 		
-		ModalMessage(nil, "You are trying to open a file belonging to an earlier "..
-			"version of WordGrinder. I can do that, but if you save the file again "..
-			"it may not work on the old version. Also, all keybindings defined in "..
-			"this file will get reset to their default values.")
+		-- This happens here because it must happen before the result of the
+		-- DocumentLoaded handlers fire, but after the DocumentUpgrade
+		-- handlers. It's not really a very elegant place for it.
 		
-		ImmediateMessage("Upgrading...")
-		FireEvent(Event.DocumentUpgrade, fileformat, FILEFORMAT)
-			
-		DocumentSet.fileformat = FILEFORMAT
-		DocumentSet.menu = CreateMenu()
-		DocumentSet:touch()
+		FireEvent(Event.RegisterAddons)
 	end
 	
 	AddEventListener(Event.DocumentLoaded, cb)
@@ -321,6 +331,8 @@ end
 
 do
 	local function cb(event, token, oldversion, newversion)
+		-- Upgrade version 1 to 2.
+		
 		if (oldversion < 2) then
 			-- Update wordcount.
 
@@ -338,6 +350,18 @@ do
 
 			DocumentSet.statusbar = true
 		end
+		
+		-- Upgrade version 2 to 3.
+		
+		if (oldversion < 3) then
+			-- Idle time defaults to 3.
+			
+			DocumentSet.idletime = 3
+			
+			-- Add the addons setting table.
+			
+			DocumentSet.addons = {}
+		end
 	end
 	
 	AddEventListener(Event.DocumentUpgrade, cb)
