@@ -1,9 +1,9 @@
--- © 2007 David Given.
+-- © 2008 David Given.
 -- WordGrinder is licensed under the BSD open source license. See the COPYING
 -- file in this distribution for the full text.
 --
--- $Id: forms.lua 5 2007-10-12 22:40:45Z dtrg $
--- $URL: $
+-- $Id: forms.lua 121 2008-09-07 15:35:27Z dtrg $
+-- $URL: https://wordgrinder.svn.sf.net/svnroot/wordgrinder/wordgrinder/src/lua/forms.lua $
 
 local int = math.floor
 local Write = wg.write
@@ -28,9 +28,26 @@ Form.Center = Form.Centre
 
 Form.Large = {}
 
+local function min(a, b)
+	if (a < b) then
+		return a
+	else
+		return b
+	end
+end
+
+local function max(a, b)
+	if (a > b) then
+		return a
+	else
+		return b
+	end
+end
+
 local function makewidgetclass(class)
 	return function(table)
 		setmetatable(table, {__index = class})
+		table.class = class
 		return table
 	end
 end
@@ -75,6 +92,7 @@ Form.Label = makewidgetclass {
 			xo = self.realwidth - GetStringWidth(self.value)
 		end
 		
+		Write(self.realx1, self.realy1, string_rep(" ", self.realwidth))
 		Write(self.realx1 + xo, self.realy1, self.value)
 	end
 }
@@ -96,17 +114,18 @@ Form.Checkbox = makewidgetclass {
 		else
 			s = "✘"
 		end
+				
+		Write(self.realx1, self.realy1, GetBoundedString(self.label, self.realwidth - 2))
 		
-		Write(self.realx1, self.realy1, s)		
+		SetBold()
+		Write(self.realx2, self.realy1, s)
+		SetNormal()
+				
 		if self.focus then
-			Goto(self.realx1, self.realy1)
+			Goto(self.realx2, self.realy1)
 		end
-		
-		Write(self.realx1 + 2, self.realy1, GetBoundedString(self.label, self.realwidth - 2))
 	end,
 	
-	["KEY_RETURN"] = checkbox_toggle,
-	["KEY_ENTER"] = checkbox_toggle,
 	[" "] = checkbox_toggle
 }
 
@@ -119,7 +138,7 @@ Form.TextField = makewidgetclass {
 	end,
 	
 	draw = function(self)
-		Write(self.realx1, self.realy1 + 1, string_rep("⎺", self.realwidth))
+		Write(self.realx1, self.realy1 + 1, string_rep("▔", self.realwidth))
 		Write(self.realx1, self.realy1, string_rep(" ", self.realwidth))
 		
 		-- If the cursor is to the left of the visible area, adjust.
@@ -145,7 +164,9 @@ Form.TextField = makewidgetclass {
 		-- Draw the visible bit of the string.
 			
 		local s = GetBoundedString(self.value:sub(self.offset), self.realwidth)
+		SetBold()
 		Write(self.realx1, self.realy1, s)
+		SetNormal()
 		
 		if self.focus then
 			Goto(self.realx1 + GetStringWidth(s:sub(1, self.cursor-self.offset)), self.realy1)
@@ -208,7 +229,7 @@ Form.TextField = makewidgetclass {
 		return "nop"
 	end,
 				
-	["KEY_DC"] = function(self, key)
+	["KEY_DELETE"] = function(self, key)
 		local v = self.value:byte(self.cursor)
 		if v then
 			local w = GetBytesOfCharacter(self.value:byte(self.cursor))
@@ -245,7 +266,24 @@ Form.Browser = makewidgetclass {
 	
 	init = function(self)
 		self.cursor = self.cursor or 1
-		self.offset = self.offset or 1
+		self.offset = self.offset or 0
+	end,
+	
+	_adjustOffset = function(self)
+		local h = self.realheight
+		
+		if (self.offset == 0) then
+			self.offset = self.cursor - int(h/2)
+		end
+
+		self.offset = min(self.offset, self.cursor)		
+		self.offset = max(self.offset, self.cursor - (h-2))
+		self.offset = min(self.offset, #self.data - (h-2)) 
+		self.offset = max(self.offset, 1)
+	end,
+	
+	changed = function(self)
+		return "nop"
 	end,
 	
 	draw = function(self)
@@ -269,16 +307,8 @@ Form.Browser = makewidgetclass {
 			Write(x+1, y+h, border)
 			Write(x+w-1, y+h, "┘")
 		end
-		
-		-- Adjust the offset.
-		
-		if (self.cursor < self.offset) then
-			self.offset = self.cursor
-		end
-		
-		if (self.cursor > (self.offset + h-2)) then
-			self.offset = self.cursor - (h-2)
-		end
+
+		self:_adjustOffset()		
 		
 		-- Draw the data.
 		
@@ -319,6 +349,7 @@ Form.Browser = makewidgetclass {
 		if (self.cursor > 1) then
 			self.cursor = self.cursor - 1
 			self:draw()
+			return self:changed()
 		end
 		
 		return "nop"
@@ -328,65 +359,80 @@ Form.Browser = makewidgetclass {
 		if (self.cursor < #self.data) then
 			self.cursor = self.cursor + 1
 			self:draw()
+			return self:changed()
 		end
 		
 		return "nop"
 	end,
 	
-	["KEY_PPAGE"] = function(self, key)
-		self.cursor = self.cursor - int(self.realheight/2)
+	["KEY_PGUP"] = function(self, key)
+		local oldcursor = self.cursor
+		self.cursor = oldcursor - int(self.realheight/2)
 		if (self.cursor < 1) then
 			self.cursor = 1
 		end
 		
-		self:draw()
+		if (self.cursor ~= oldcursor) then
+			self:draw()
+			return self:changed()
+		end
+		return "nop"
 	end,
 	
-	["KEY_NPAGE"] = function(self, key)
-		self.cursor = self.cursor + int(self.realheight/2)
+	["KEY_PGDN"] = function(self, key)
+		local oldcursor = self.cursor
+		self.cursor = oldcursor + int(self.realheight/2)
 		if (self.cursor > #self.data) then
 			self.cursor = #self.data
 		end
 		
-		self:draw()
+		if (self.cursor ~= oldcursor) then
+			self:draw()
+			return self:changed()
+		end
+		return "nop"
 	end,
 }
 
 local standard_actions =
 {
 	["KEY_UP"] = function(dialogue, key)
-		local f = dialogue.focus - 1
-		while (f ~= dialogue.focus) do
-			if (f == 0) then
-				f = #dialogue
+		if dialogue.focus then
+			local f = dialogue.focus - 1
+			while (f ~= dialogue.focus) do
+				if (f == 0) then
+					f = #dialogue
+				end
+	
+				local widget = dialogue[f]
+				if widget.focusable then
+					dialogue.focus = f
+					return "redraw"
+				end
+			
+				f = f - 1
 			end
-
-			local widget = dialogue[f]
-			if widget.focusable then
-				dialogue.focus = f
-				return "redraw"
-			end
-		
-			f = f - 1
 		end
 		
 		return "nop"
 	end,
 	
 	["KEY_DOWN"] = function(dialogue, key)
-		local f = dialogue.focus + 1
-		while (f ~= dialogue.focus) do
-			if (f > #dialogue) then
-				f = 1
+		if dialogue.focus then
+			local f = dialogue.focus + 1
+			while (f ~= dialogue.focus) do
+				if (f > #dialogue) then
+					f = 1
+				end
+	
+				local widget = dialogue[f]
+				if widget.focusable then
+					dialogue.focus = f
+					return "redraw"
+				end
+			
+				f = f + 1
 			end
-
-			local widget = dialogue[f]
-			if widget.focusable then
-				dialogue.focus = f
-				return "redraw"
-			end
-		
-			f = f + 1
 		end
 		
 		return "nop"
