@@ -1,9 +1,9 @@
--- © 2007 David Given.
+-- © 2008 David Given.
 -- WordGrinder is licensed under the BSD open source license. See the COPYING
 -- file in this distribution for the full text.
 --
--- $Id: navigate.lua 45 2008-01-13 06:38:47Z dtrg $
--- $URL: https://wordgrinder.svn.sourceforge.net/svnroot/wordgrinder/wordgrinder/src/lua/navigate.lua $
+-- $Id: navigate.lua 121 2008-09-07 15:35:27Z dtrg $
+-- $URL: https://wordgrinder.svn.sf.net/svnroot/wordgrinder/wordgrinder/src/lua/navigate.lua $
 
 local int = math.floor
 local GetBytesOfCharacter = wg.getbytesofcharacter
@@ -19,11 +19,13 @@ local table_concat = table.concat
 
 function Cmd.GotoBeginningOfWord()
 	Document.co = 1
+	QueueRedraw()
 	return true
 end
 
 function Cmd.GotoEndOfWord()
 	Document.co = Document[Document.cp][Document.cw].text:len() + 1
+	QueueRedraw()
 	return true
 end
 
@@ -37,8 +39,19 @@ function Cmd.GotoEndOfParagraph()
 	return Cmd.GotoEndOfWord()
 end
 
+function Cmd.GotoBeginningOfDocument()
+	Document.cp = 1
+	return Cmd.GotoBeginningOfParagraph()
+end
+
+function Cmd.GotoEndOfDocument()
+	Document.cp = #Document
+	return Cmd.GotoEndOfParagraph()
+end
+
 function Cmd.GotoPreviousParagraph()
 	if (Document.cp == 1) then
+		QueueRedraw()
 		return false
 	end
 	
@@ -52,6 +65,7 @@ end
 	
 function Cmd.GotoNextParagraph()
 	if (Document.cp == #Document) then
+		QueueRedraw()
 		return false
 	end
 
@@ -73,6 +87,7 @@ end
 
 function Cmd.GotoPreviousWord()
 	if (Document.cw == 1) then
+		QueueRedraw()
 		return false
 	end
 	
@@ -86,6 +101,7 @@ end
 function Cmd.GotoNextWord()
 	local p = Document[Document.cp]
 	if (Document.cw == #p) then
+		QueueRedraw()
 		return false
 	end
 	
@@ -168,6 +184,20 @@ function Cmd.InsertStringIntoWord(c)
 	return true
 end
 
+function Cmd.InsertStringIntoParagraph(c)
+	local first = true
+	for word in c:gmatch("[^%s]+") do
+		if not first then
+			Cmd.SplitCurrentWord()
+		end
+		
+		Cmd.InsertStringIntoWord(word)
+		
+		first = false
+	end
+	return true
+end
+
 function Cmd.SplitCurrentWord()
 	local paragraph = Document[Document.cp]
 	local word = paragraph[Document.cw]
@@ -495,7 +525,7 @@ function Cmd.ChangeParagraphStyle(style)
 	
 	DocumentSet:touch()
 	QueueRedraw()
-	return true
+	return Cmd.UnsetMark()
 end
 
 function Cmd.ToggleMark()
@@ -532,53 +562,6 @@ function Cmd.ChangeDocument(name)
 	return true
 end
 
-function Cmd.RenameDocument(name)
-	if not name then
-		name = PromptForString("Change name of current document", "Please enter the new document name:", Document.name)
-		if not name then
-			return false
-		end
-	end
-		
-	if (name ~= Document.name) and DocumentSet.documents[name] then
-		ModalMessage("Name in use", "Sorry! There's already a document with that name in this document set.")
-		return false
-	end
-
-	DocumentSet:renameDocument(Document.name, name)
-	QueueRedraw()
-	return true
-end
-
-function Cmd.DeleteDocument()
-	if not PromptForYesNo("Delete this document?", "Are you sure you want to delete this document? It will be removed from the current document set, and will be gone forever.") then
-		return false
-	end
-	
-	DocumentSet:deleteDocument(Document.name)
-	QueueRedraw()
-	return true
-end
-
-function Cmd.AddBlankDocument(name)
-	if not name then
-		name = PromptForString("Name of new document?", "Please enter the new document name:")
-		if not name or (name == "") then
-			return false
-		end
-	end
-
-	if DocumentSet.documents[name] then
-		ModalMessage("Name in use", "Sorry! There's already a document with that name in this document set.")
-		return false
-	end
-
-	DocumentSet:addDocument(CreateDocument(), name)
-	DocumentSet:setCurrent(name)
-	QueueRedraw()
-	return true
-end
-
 function Cmd.Cut()
 	return Cmd.Copy(true) and Cmd.Delete()
 end
