@@ -2,7 +2,7 @@
 -- WordGrinder is licensed under the BSD open source license. See the COPYING
 -- file in this distribution for the full text.
 --
--- $Id: utils.lua 137 2008-09-08 21:47:32Z dtrg $
+-- $Id: utils.lua 145 2008-11-03 18:30:16Z dtrg $
 -- $URL: https://wordgrinder.svn.sf.net/svnroot/wordgrinder/wordgrinder/src/lua/utils.lua $
 
 function max(a, b)
@@ -60,3 +60,23 @@ function Leafname(filename)
 	end
 	return filename
 end
+
+--- Produces an exception traceback.
+--
+-- @param e                  the error
+-- @return                   the trace, as a string
+
+function Traceback(e)
+	local i = 1
+	local s = {"Exception: "..e}
+	while true do
+		local t = debug.getinfo(i)
+		if not t then
+			break
+		end
+		s[#s+1] = t.short_src .. ":" .. t.currentline 
+		i = i + 1
+	end
+
+	return table.concat(s, "\n")	
+end
